<?php

/**
 * vim:et:sw=4:ts=4
 * 
 * An interface between the MagicSpam control files and PHP. 
 *
 * Copyright (c) 2008 LinuxMagic Inc., All Rights Reserved.
 *
 * $Id: class.MagicSpamControl.php 1161 2009-09-11 17:08:39Z shaun $
 *
 */

class MagicSpamControl {
    /**
     * Directory which holds the control files.
     *
     * @var string
     */
    private static $controlDir = CONTROLS;

    /**
     * Disable creation of the constructor.
     */
    private function __construct() {
    }

    /**
     * Grabs the value from the control file corresponding to the specified 
     * name. If there was a problem reading the file, or it doesn't exist
     * the default value is returned.
     *
     * TODO: Might be an idea to have an array of default values, rather 
     *       than passing them in each time.
     *
     * @param $name Name of the control.
     * @param $default Value to return if there was a problem with the control
     *                 file.
     *
     * @return Value contained in control file, or $default if there was a
     *         problem reading the control file.
     */
    static public function get($name, $default) {
        $data = self::read($name);
        if ($data === false) {
            return $default;
        }

        return $data;
    }

    /**
     * Reads a multi-line control file. Each value is placed in an array.
     *
     * @param $name Control name.
     * @param $default Default value if control file is empty or can't be 
     *                 accessed.
     */
    static public function getMulti($name, $default) {
        $data = self::read($name);
        if ($data === false) {
            return $default;
        }

        $ar = explode("\n", $data);
        if (empty($ar)) {
            return $default;
        }

        return $ar;
    }

    /**
     * Reads the control file and returns the data.
     *
     * @param string $name Control name
     * @return string Data from file or false if the control file does not
     *                exist or we had trouble reading it.
     */
    private static function read($name) {
        $controlFile = MagicSpamControl::$controlDir . "/" . $name;
        if (!file_exists($controlFile)) {
            return false;
        }

        $data = file_get_contents($controlFile);
        if ($data === false) {
            return false;
        }

        return $data;
    }

    /**
     * Writes the specified value to the control.
     *
     * @param string $name Control name
     * @param string $value Value to write
     */
    static public function set($name, $value) {
        self::write($name, $value);
    }

    /**
     * Writes the specified values to the control.
     *
     * @param string $name Control name
     * @param array $values Values to write
     */
    static public function setMulti($name, $values) {
        self::write($name, implode("\n", $values));
    }

    /**
     * Writes the data to the specified control file. If is not writable
     * or the directory it is contained within is not, then it will throw
     * an exception. Exceptions will also be thrown if there is a problem
     * opening or writing to the file.
     *
     * @param string $name Control name
     * @param string $data Data to write to control
     */
    static private function write($name, $data) {
        $controlFile = self::$controlDir . "/" . $name;

        if (file_exists($controlFile) && !is_writable($controlFile)) {
            throw new Exception("Control file is not writable. [" . $controlFile . "]");
        } else if (!is_writable(dirname($controlFile))) {
            throw new Exception("Control directory is not writable. [" . dirname($controlFile) . "]");
        }

        $fp = fopen($controlFile, "w+b");
        if (!$fp) {
            throw new Exception("Cannot open control file [" . $controlFile . "]");
        }

        if (fwrite($fp, $data) === false) {
            throw new Exception("Cannot write to control file [" . $controlFile . "]");
        }

        fclose($fp);
    }
}

?>
