<?php

/**
 * vim:et:sw=4:ts=4
 * 
 * A simple API between the MagicSpam exemption lists and the interface. It
 * uses the magicspam-exemption wrapper utility.
 * 
 * Copyright (c) 2008 LinuxMagic Inc., All Rights Reserved.
 * 
 * $Id: class.MagicSpamExemption.php 1030 2009-07-17 16:29:09Z shaun $
 * 
 */

class MagicSpamExemption {
    /**
     * Singleton object
     * @var MagicSpamExemption
     */
    private static $object = false;

    /**
     * Location of the list manager
     * @var string
     */
    private static $list_mngr_bin = LIST_MANAGER;

    /**
     * Create a new MagicSpamExemption object using the specified wrapper 
     * utility.
     */
    private function __construct() {
        $this->lastError = false;
        $this->message = null;

        if (!is_executable(self::$list_mngr_bin)) {
            throw new Exception("List manager not found");
        }
    }
    
    /**
     * Get our exemption object. 
     */
    public function getInstance() {
        if (self::$object === false) {
            self::$object = new MagicSpamExemption;
        }

        return self::$object;
    }
    
    /**
     * Run the wrapper utility passing the specified arguments. If the utility
     * exits with a zero (0), the lastError variable of the object will be set
     * to the error message. The error message is grabbed from the first line
     * of output from the utility.
     * 
     * @param string $args Arguments to pass to the wrapper utility.
     * @param int &$exitCode Reference to a variable to store the exit code of the 
     *                       wrapper utility.
     * @return Array of lines sent to STDOUT from the wrapper utility.
     */
    private function exec($args, &$exitCode) {
        $cmd = self::$list_mngr_bin . " " . $args;
        exec($cmd, $ar, $exitCode);

        if ($exitCode != 0 && isset($ar[0])) {
            $this->lastError = implode("\n",$ar) . " [{$cmd}]";
        }

        return $ar;
    }
    
    /**
     * Gets the last error generated by the wrapper utility.
     * 
     * @return String of the last error.
     */
    public function getLastError() {
        $err = $this->lastError;
        $this->lastError = false;
        
        return $err;
    }

    /**
     * Gets the last status 'message'
     */
    public function getLastMessage() {
        $lmsg = $this->message;
        $this->message = null;
        return $lmsg;
    }
    
    /**
     * Get a list of exempt users.
     * 
     * @return Array of exempt users.
     */
    public function getUsers() {
        $list = $this->exec("-u -l", $ret);
        
        return ( $ret == 0 ? $list : array() );
    }
    
    /**
     * Get a list of exempt IP addresses.
     * 
     * @return Array of exempt IP addresses.
     */
    public function getIPs() {
        $list = $this->exec("-i -l", $ret);
        
        return ( $ret == 0 ? $list : array() );
    }

    /**
     * Get a list of the blacklisted ips
     *
     * @return Array of blacklisted IP addresses.
     */
    public function getBLIPs() {
        $list = $this->exec("-b -l", $ret);
        return ($ret == 0 ? $list: array());
    }
    /**
     * Get a list of whitelist entries.
     */
    public function getWhiteListEntries() {
        $list = $this->exec("-f -l", $ret);

        return ( $ret == 0 ? $list : array() );
    }

    /**
     * Get a list of blacklist from entries.
     */
    public function getBlackListEntries() {
        $list = $this->exec("-k -l", $ret);
        return ( $ret == 0 ? $list : array() );
    }
    
    /**
     * Append a value to one of the exemption lists. The list which the value
     * is added to is determined by the wrapper utility.
     *
     * @param string $value Value to append to exemption list.
     * @return true on success; false on failure.
     */
    public function add($value,$type) {
        switch($type) {
            case "user":
                return $this->add_user($value);
                break;
            case "ip":
                return $this->add_ip($value);
                break;
            case "ipblacklist":
                return $this->add_ipbl($value);
                break;
            case "whitelist":
                return $this->add_whitelist($value);
                break;
            case "fromblacklist":
                return $this->add_fromblacklist($value);
                break;
            default:
                return false;
                break;
        }
    }
   
    /**
     * Append a value to the exempt users list
     * NOTE: 2008-10-27: modified from generic 'add' to work
     *                   with new magicspam-listmgr utility.
     * Shaun <shaun@linuxmagic.com>
     */
    private function add_user($value) {
        // safety check - no domain wildcards permitted.
        $parts = explode('@',$value);
        if (!is_array($parts) || count($parts) != 2) {
            $this->lastError = "Invalid entry. Must be in the form user@domain";
            return false;
        }
        if ($parts[1] == '*') {
            $this->lastError = "Invalid entry. Domain wildcards are not supported for safety reasons";
            return false;
        }
        //Test the entry for attempted wildcard entries....
        if (strchr($value,"*")) {
            $this->message = "WARNING: while '*' wildcards are supported, they are not generally recommended";
        }

        //TODO: actually do something with $err...
        $err = $this->exec("-u -a " . escapeshellarg($value),$ret);

        return ( $ret == 0 );
    }

    /**
     * Append a value to the exempt ip list
     * NOTE: 2008-10-27: modified from generic 'add' to work
     *                   with new magicspam-listmgr utility.
     * Shaun <shaun@linuxmagic.com>
     */
    private function add_ip($value) { 
        //TODO: actually do something with $err....
        $err = $this->exec("-i -a " . escapeshellarg($value),$ret);

        return ( $ret == 0 );
    }

    /**
     * Append a value to the blacklist ip list.
     */
    private function add_ipbl($value) {
        $err = $this->exec("-b -a " .escapeshellarg($value),$ret);
        return ($ret == 0);
    }
    /**
     * Append a value to the whitelist
     */
    private function add_whitelist($value) {
        // safety check - no domain wildcards permitted.
        $parts = explode('@',$value);
        if (!is_array($parts) || count($parts) != 2) {
            $this->lastError = "Invalid entry. Must be in the form user@domain";
            return false;
        }
        if ($parts[1] == '*') {
            $this->lastError = "Invalid entry. Domain wildcards are not supported for safety reasons";
            return false;
        }

        //Test the entry for attempted wildcard entries....
        if (strchr($value,"*")) {
            $this->message = "WARNING: while '*' wildcards are supported, they are not generally recommended";
        }

        $err = $this->exec("-f -a " . escapeshellarg($value),$ret);

        return ( $ret == 0 );
    }

    private function add_fromblacklist($value) {
        //Test the entry for attempted wildcard entries....
        if (strchr($value,"*")) {
            $this->lastError = "Invalid entry. wildcards are NOT permitted for a blacklist entry";
            return false;
        }

        $err = $this->exec("-k -a " . escapeshellarg($value),$ret);

        return ( $ret == 0 );
    }

    /**
     * Remove a value from one of the exemption lists. the list which the value
     * is removed from is determined by the wrapper utility.
     *
     * @param string $value Value to remove from the exemption list.
     * @return true on success; false on failure.
     */
    public function remove($value,$type) {
        switch ($type) {
            case "user":
                return $this->remove_user($value);
                break;
            case "ip":
                return $this->remove_ip($value);
                break;
            case "ipblacklist":
                return $this->remove_ipbl($value);
                break;
            case "whitelist":
                return $this->remove_whitelist($value);
                break;
            case "fromblacklist":
                return $this->remove_fromblacklist($value);
                break;
            default:
                return false;
                break;
        }
    }

    /**
     * Remove a value from the exempt ip list
     * NOTE: 2008-10-27: modified from generic 'remove' to work
     *                   with new magicspam-listmgr utility
     *
     * Shaun <shaun@linuxmagic.com>
     */
    private function remove_ip($value) {
        $this->exec("-i -d " . escapeshellarg($value), $ret);

        return ( $ret == 0 );
    }

    /**
     * Remove a value from the exempt user list
     * NOTE: 2008-10-27: modified from generic 'remove' to work
     *                   with new magicspam-listmgr utility
     *
     * Shaun <shaun@linuxmagic.com>
     */
    private function remove_user($value) {
        $this->exec("-u -d " . escapeshellarg($value), $ret);

        return ( $ret == 0);
    }

    /**
     * Remove a value from the whitelist
     */
    private function remove_whitelist($value) {
        $this->exec("-f -d " . escapeshellarg($value), $ret);

        return ( $ret == 0 );
    }

    /**
     * Remove a value from the from blacklist
     */
    private function remove_fromblacklist($value) {
        $this->exec("-k -d " . escapeshellarg($value), $ret);

        return ( $ret == 0 );
    }

    /**
     * Remove a value from the ip blacklist.
     */
    private function remove_ipbl($value) {
        $this->exec("-b -d " . escapeshellarg($value),$ret);
        return ($ret == 0);
    }
}

?>
