<?php

/* vim:et:sw=4:ts=4
 *
 * This is a container to access MagicSpam license information.  
 * This class allows for the reading of MagicSpan License information in conjunction with information 
 * pulled from license manager based on the license code.  Licenses are described using 5 variables: 
 * $state, $code, $expiry, $nextupdate, $requester.
 *
 * Copyright (c) 2008 LinuxMagic Inc., All Rights Reserved.
 *
 * $Id: class.MagicSpamLicense.php 697 2009-02-25 23:14:34Z shaun $
 *
 */

 
class MagicSpamLicense {
    /**
     * This holds our singleton license object
     *
     * @var MagicSpamLicense
     * @access private
     */
    static private $object = false;

    /**
     * Location of the license manager wrapper binary
     *
     * @var string
     * @access private
     */
    static private $license_manager_bin = LICENSE_MANAGER;

    /**
     * MagicSpam license state codes
     *
     * @var array
     * @access private
     */
    static private $state_codes = array(
        0 => "valid", 
        1 => "pending", 
        2 => "expired", 
        3 => "new"
    );

    /**
     * MagicSpam license key
     *
     * @var string
     * @access private
     */
    private $code;

    /**
     * MagicSpam license key expiry timestamp
     *
     * @var timestamp
     * @access private
     */
    private $expiry;

    /**
     * MagicSpam next update timestamp
     *
     * @var nextupdate
     * @access private
     */
    private $nextupdate;

    /**
     * MagicSpam license requester
     * @var requester
     * @access private
     */
    private $requester;

    /**
     * MagicSpam license key status code. See $status_codes.
     *
     * @var integer
     * @access private
     */
    private $state;

    /**
     * Disable constructor
     */
    private function __construct() {
        if (!is_executable(self::$license_manager_bin)) {
            throw new Exception("License Manager not found. (" . self::$license_manager_bin . ")");
        }

        $this->code = false;
        $this->expiry = false;
        $this->nextupdate = false;
        $this->enabled = false;
        $this->requester = false;
        $buffer = "";

        try {
            // load it up...
            $this->load();
            $this->state = self::exec("state",$buffer);
        } catch (Exception $e) {
            $this->state = self::state("new");
        }
    }

    /**
     * Get our license object.
     */
    static function getInstance() {
        if (self::$object === false) {
            self::$object = new MagicSpamLicense;
        }

        return self::$object;
    }

    /**
     * Converts a state string to the corresponding integer value and vice-versa.
     */
    static function state($value) {
        if (is_numeric($value)) {
            return self::$state_codes[$value];
        } else if (is_string($value)) {
            return array_search($value, self::$state_codes);
        }

        return false;
    }

    /**
     * Runs the given action against the license manager binary.
     *
     * @param string $action Arguments to pass to license manager.
     *        string buffer $buffer any stdout output to buffer.
     * @return integer 
     */
    static private function exec($action, &$buffer) {
        $cmd = self::$license_manager_bin . " $action";
        $fh = popen($cmd, "r");
        if (!$fh) {
            throw new Exception("Failed to communicate with license manager.");
        }

        $result = "";
        while ($line = fgets($fh)) {
            $result .= $line;
        }

        $result = trim($result);
        $buffer = $result;
        $exit_code = pclose($fh);
        if ($exit_code == 255) {
            throw new Exception($result);
        }

        return $exit_code;
    }

    /**
     * Updates the active MagicSpam license key
     * 
     * @param $code MagicSpam license key
     * @return int MagicSpam license key state for given code
     */
    public function update($code = false) {
        $cmd = false;
        $buffer = "";
        if ($code === false) {
            $cmd = "update";
        } else if (!empty($code)) {
            // TODO: maybe above needs to be a regular expression check?
            $cmd = "update " . escapeshellarg($code);
        }
        if ($cmd === false) {
            throw new Exception("Poorly formatted license key given.");
        }
        $result = self::exec($cmd, $buffer);
        if ($result != 0) {
            throw new Exception($buffer);
        }
        $this->load();
        $this->state = self::exec("state",$buffer);
    }

    /**
     * Loads a MagicSpam license key.  
     * Communicates with the license manager to set the private class variables: 
     * $code, $expiry, $nextupdate, $requester, $got_data.  
     * Return is undefined.
     */
    public function load() {
        $buffer = "";
        $result = self::exec("data", $buffer);
        if ($result != 0) {
            throw new Exception("Exec failed [{$result}] " . $buffer);
        }
        $linearray = explode("\n",$buffer);
        $got_data = false;
        foreach($linearray as $line) {
            $resarray = explode(":", $line);
        
            if (count($resarray) == 6) {
                $this->code = $resarray[2];
                $this->expiry = $resarray[0];
                $this->nextupdate = $resarray[4];
                $this->requester = $resarray[5];
                $got_data = true;
            }
        }
        if (!$got_data) {
            throw new Exception("Problem communicating with license manager. [{$buffer}]");
        }
    }

    /**
     * Accessor to check if state is set to valid
     *
     * @return true if state is valid
     */
    public function isValid() {
        return $this->state === self::state("valid");
    }

    /**
     * Accessor to check if state is set to expired
     *
     * @return true if state is expired
     */
    public function isExpired() {
        return $this->state === self::state("expired");
    }
    
    /**
     * Accessor to check if state is set to pending
     *
     * @return true if state is pending
     */
    public function isPending() {
        return $this->state === self::state("pending");
    }

    /**
     * Accessor to check if state is set to new
     *
     * @return true if state is new
     */
    public function isNew() {
        return $this->state === self::state("new");
    }

    /**
     * Accessor to retrieve the code
     *
     * @return string containing the code
     */
    public function getCode() {
        return $this->code;
    }

    /**
     * Accessor to retrieve the expiry date
     *
     * @return date string formatted m-d-Y
     */
    public function getExpiry() {
        if (isset($this->expiry) && is_numeric($this->expiry)) {
            return date('F j, Y', $this->expiry);
        }
        return false;
    }

    /**
     * Accessor to retrieve the next update date
     *
     * @return date string formatted m-d-Y if next update exists or false otherwise
     */
    public function getNextUpdate() 
    {
        if (isset($this->nextupdate) && is_numeric($this->nextupdate)) {
            return date('F j, Y', $this->nextupdate);
        }
        return false;
    }

    /**
     * Accessor to retrieve requester
     *
     * @return string if set, false otherwise
     */
    public function getRequester()
    {
        if (isset($this->requester) && strlen($this->requester)) {
            return $this->requester;
        }
        return false;
    }
}

?>
