<?php
/**
 * vim:et:sw=4:ts=4
 * 
 * A simple interface handler to ensure valid pages are outputed to it's child class while rendering tabs.  
 * A series of checks our made before initiating our MagicSpamPage class to finalize output to our interface.
 *  
 * Copyright (c) 2008 LinuxMagic Inc., All Rights Reserved.
 * 
 * $Id: class.MagicSpamExemption.php 1030 2009-07-17 16:29:09Z shaun $
 * 
 */

require_once('class.MagicSpamLicense.php');
require_once('class.MagicSpamControl.php');

/**
 * Page handler for ensuring page validity and rendering navigation.
 *
 * @params array $page_init name of pages to be rendered 
 */
class MSInterface {

    /**
     * An array of pages to be outputted  with special characters converted.
     *
     * @var array $pages
     */
    protected $pages;

    /**
     * A pointer to the active page.  Is set to false if no page is active.
     *
     * @var mixed
     */
    protected $activePage;

    /**
     * Constructor wrapper that accepts an array of pages to be rendered.  
     * Sets $this->page to passed array after passing each page through init() 
     * and $this->activePage to equal false.
     *
     * @params array $page_init
     */
    public function __construct($page_init) {
        $this->pages = array();
        foreach ($page_init as $page) {
            $this->pages[$page] = intl($page);
        }
        $this->activePage = false;
    }

    /**
     * Page handler performing a series of checks before creating the child MagicSpamPage object.  
     * If run is called with no argument then license and admin email is checked for validity, 
     * upon failure the settings page is shown, upon success first page on $this->page list.
     * Otherwise argument is accepted and checked for validity, upon success activePage is 
     * set to argument and a child MagicSpamPage object is created, prepared, and rendered, and
     * navigation tabs are outputted. Upon failure an exception is thrown.
     *
     * @params string $page an argument of page to run.
     */
    public function run($page = false) {
        if ($page === false) {
            $license = MagicSpamLicense::getInstance();
            $adminmail = MagicSpamControl::get("admin_email","");

            if (!$license->isValid() || $adminmail == "") {
                // go to the settings page if license is not valid
                $page = "settings";
            } else {
                // use the first page on the list if license is valid
                $page = key($this->pages);
            }
        } else {
            if (!array_key_exists($page, $this->pages)) {
                throw new Exception(intl('invalidpage'));
            }
        }

        $file = PAGES . $page . ".php";
        if (!file_exists($file)) {
            throw new Exception(intl('invalidpage') . " :" . ($file));
        }

        if (!include($file)) {
            throw new Exception(intl('invalidpage'));
        }

        $class = "MagicSpamPage_" . $page;
        if (!class_exists($class)) {
            throw new Exception(intl('invalidpage'));
        }

        // set the requested page to the active page
        $this->activePage = $page;

        // create our page object
        $pageObj = new $class;

        // prepare our page, allow the child class to prepare as well
        $this->prepare($pageObj);

        // pass the page object to our child class to handle output
        $this->render($pageObj);
    }

    /**
     * Calls the prepare function of the passed MagicSpamPage object
     * 
     * @params MagicSpamPage $page
     */
    protected function prepare(MagicSpamPage $page) {
        $page->prepare();
    }

    /**
     * Creates tabs and calls the render function of the passed MagicSpamPage object.
     *
     * @params MagicspamPage $page
     */
    protected function render(MagicSpamPage $page) {
        // let's render some tabs....
        echo "<div id=\"navigation\"><ul>\n";
        foreach ($this->pages as $pageID => $label) {
            // Give current tab the "selected" class
            if ($this->activePage == $pageID) {
                $selected = " class=\"tabselected\"";
            } else {
                $selected = " class=\"tab\"";
            }
            //if ($_GET && isset($_GET['page']) && ($_GET['page'] == $pageID)) $selected = " class=\"selected\"";
            //else $selected = false;
            echo <<<END
<li>
    <a href="?page={$pageID}" {$selected}>
        <span>{$label}</span>
    </a>
</li>
END;
        }
        echo "</ul></div>\n";
        if ($page) {
            $page->render();
        }
    }

}

/**
 * Converts special characters to HTML friendly syntax.
 *
 * @param string $key
 */
function intl($key) {
    global $ms_words;
    $ret = "";
    
    $charset = array(
         "" => "&aacute;"
        ,"" => "&Aacute;"
        ,"" => "&agrave;"
        ,"" => "&Agrave;"
        ,"" => "&acirc;"
        ,"" => "&Acirc;"
        ,"" => "&aring;"
        ,"" => "&Aring;"
        ,"" => "&atilde;"
        ,"" => "&Atilde;"
        ,"" => "&auml;"
        ,"" => "&Auml;"
        ,"" => "&aelig;"
        ,"" => "&Aelig;"
        ,"" => "&ccedil;"
        ,"" => "&Ccedil;"
        ,"" => "&eacute;"
        ,"" => "&Eacute;"
        ,"" => "&egrave;"
        ,"" => "&Egrave;"
        ,"" => "&ecirc;"
        ,"" => "&Ecirc;"
        ,"" => "&euml;"
        ,"" => "&Euml;"
        ,"" => "&iacute;"
        ,"" => "&Iacute;"
        ,"" => "&igrave;"
        ,"" => "&Igrave;"
        ,"" => "&icirc;"
        ,"" => "&Icirc;"
        ,"" => "&iuml;"
        ,"" => "&Iuml;"
        ,"" => "&ntilde;"
        ,"" => "&Ntilde;"
        ,"" => "&oacute;"
        ,"" => "&Oacute;"
        ,"" => "&ograve;"
        ,"" => "&Ograve;"
        ,"" => "&ocirc;"
        ,"" => "&Ocirc;"
        ,"" => "&oslash;"
        ,"" => "&Oslash;"
        ,"" => "&otilde;"
        ,"" => "&Otilde;"
        ,"" => "&ouml;"
        ,"" => "&Ouml;"
        ,"" => "&szlig;"
        ,"" => "&uacute;"
        ,"" => "&Uacute;"
        ,"" => "&ugrave;"
        ,"" => "&Ugrave;"
        ,"" => "&ucirc;"
        ,"" => "&Ucirc;"
        ,"" => "&uuml;"
        ,"" => "&Uuml;"
        ,"" => "&yuml;"
        ,"" => "&iquest;"
        ,"" => "&laquo;"
        ,"" => "&raquo;"
    );

    if ( isset($ms_words) && is_array($ms_words)
        && isset($key) && isset($ms_words[$key]) ) {
        $ret = $ms_words[$key];

        foreach ($charset as $schar => $repl) {
            $ret = preg_replace('/'.$schar.'/',$repl,$ret);
        }
    }
    return $ret;
}

?>
