<?php

/**
 * RCPTCounter is a simple method for obtaining stored data
 * counts from the magicspam engine.
 *
 * This file is part of the MagicSpam(TM) product line and is
 * Copyright (c) LinuxMagic Inc. 2009 All Rights Reserved.
 *
 * Author: Shaun A. Johnson <shaun@linuxmagic.com>
 * $Id$
 */

define('RCPT_TYPE_TOTAL',0);
define('RCPT_TYPE_HAM',1);
define('RCPT_TYPE_SPAM',2);
define('RCPT_TYPE_EXEMPT',3);
define('RCPT_TYPE_ALL',4);

/**
 * Pulls counter data based on a month argument.
 *
 * @param string $month
 *
 */
class RCPTCounter
{
    /**
     *  The data file path.
     *  @var string $_m_strFilePath
     */
    var $_m_strFilePath;

    /**
     *  Array to hold the 'last month' of data.
     *  @var array $_m_arrData;
     */
    var $_m_arrData;

    /**
     *  Buffer for any incidental errors that come up.
     *  @var string $_m_strError;
     */
    var $_m_strError;

    /**
     * String to hold the month 
     * $var string $month
     */
    var $_month;

    //! PHP4 compatibility wrapper
    function RCPTCounter() {
        // register destructor
        register_shutdown_function(array(&$this,'__destruct'));

        // register constructor
        $argcv = func_get_args();
        call_user_func_array(array(&$this,'__construct',$argcv));
    }

    /**
     * Constructs object based on month passed and file contents.
     */
    function __construct($month) {
        $this->_month = $month;
        $this->_m_strFilePath = sprintf("%s.%02d",COUNTER_PATH,$month);
        $this->_m_arrData = array();
    }

    //! destructor
    function __destruct() {
        // nothing to see here folks.. move along now.
    }

    /**
     * string error()
     *  get the last buffered error message
     *
     *  @return string error message
     */
    function error() {
        return $this->_m_strError;
    }

    /**
     * bool load()
     *  attempt to load the data from the file.
     *
     *  @return bool TRUE if success
     */
    function load() {
        if (!file_exists($this->_m_strFilePath)) {
            $this->_m_strError = "File {$this->_m_strFilePath} does not exist!";
            return false;
        }
        $fh = fopen($this->_m_strFilePath,"rb");
        if ($fh === FALSE) {
            $this->_m_strError = "Error opening {$this->_m_strFilePath}";
            return false;
        }
        // 31 day records, 4 integers per record.
        $datsize = 31 * 4 * 4;
        $datsize += 4; // account for the 'sent' timestamp field.
        unset($this->_m_arrData);
        $this->_m_arrData = array();
        // is the file the expected size? if not, we *must* conclude its corrupt.
        fseek($fh,0,SEEK_END);
        if (ftell($fh) != $datsize) {
            $this->_m_strError = "File {$this->_m_strFilePath} is not a valid counter data file: " . ftell($fh) . " : " . $datsize;
            fclose($fh);
            return false;
        }
        fseek($fh,0,SEEK_SET);
        $bytes_left = $datsize;
        $counter = 0;

        // grab the 'sent' flag... this may change from bool to uint32_t (timestamp)
        $data = fread($fh,4);
        if ($data != 0) {
            $sent = true;
        } else {
            $sent = false;
        }
        $bytes_left -= 4;
        $this->_m_arrData['sent'] = $sent;
        while ($bytes_left > 0) {
            // timestamp
            $data = fread($fh,4); // grab 'intsize bytes - timestamp
            $ts = unpack("L",$data); // unpack it - unsigned long
            $bytes_left -= 4;
            $data = fread($fh,4); // grab 'intsize' bytes - ham
            $ham = unpack("L",$data); // unpack it - unsigned long
            $bytes_left -= 4;
            $data = fread($fh,4); // grab - spam
            $spam = unpack("L",$data);
            $bytes_left -= 4;
            $data = fread($fh,4);
            $exempt = unpack("L",$data);
            $bytes_left -= 4;

            $this->_m_arrData[$counter] = array();
            $this->_m_arrData[$counter]['timestamp'] = $ts[1];
            $this->_m_arrData[$counter]['ham'] = $ham[1];
            $this->_m_arrData[$counter]['spam'] = $spam[1];
            $this->_m_arrData[$counter]['exempt'] = $exempt[1];
            $counter++;
        }
        fclose($fh);
        return true;
    }

    /**
     * mixed get_value (month,day,type)
     *
     *  load a value for a given month and day.
     *  returns FALSE if it doesn't exist.
     *  arguments are integers where....
     *       January = 1
     *       February = 2 ... etc....
     *
     *       1st of the month = 1
     *       2nd of the month  = 2 etc...
     *
     *       RCPT_TYPE_{TOTAL,HAM,SPAM,EXEMPT,ALL}
     */
    function get_value($day,$type) {
        // since the array is based on a zero indexed count...
        $day--;
        if (isset($this->_m_arrData)) {
            if (isset($this->_m_arrData[$day])) {
                $ts = $this->_m_arrData[$day]['timestamp'];
                if ($this->_month == date("n",$ts)) {
                    switch($type) {
                    case RCPT_TYPE_TOTAL:
                        $total = $this->_m_arrData[$day]['ham'];
                        $total += $this->_m_arrData[$day]['spam'];
                        $total += $this->_m_arrData[$day]['exempt'];
                        return $total;
                        break;
                    case RCPT_TYPE_HAM:
                        return $this->_m_arrData[$day]['ham'];
                        break;
                    case RCPT_TYPE_SPAM:
                        return $this->_m_arrData[$day]['spam'];
                        break;
                    case RCPT_TYPE_EXEMPT:
                        return $this->_m_arrData[$day]['exempt'];
                        break;
                    case RCPT_TYPE_ALL:
                        return $this->_m_arrData[$day];
                        break;
                    }
                }
            }
        }
        return 0;
    }

    /**
     * Get the 'date' value for a given index in the data array...
     * @return mixed Timestamp if found, FALSE if not found
     */
    function get_date($idx) {
        if (isset($this->_m_arrData)) {
            if (isset($this->_m_arrData[$idx])) {
                return $this->_m_arrData[$idx]['timestamp'];
            }
        }
        return 0;
    }
}
?>

