<?php

/* vim:et:sw=4:ts=4
 *
 * Copyright (c) 2008 LinuxMagic Inc., All Rights Reserved.
 *
 * $Id: class.MagicSpamSpamRulesSection.php 1161 2009-09-11 17:08:39Z shaun $
 *
 * This file controls the output and interaction with the various spam rules.
 */

require_once("class.MagicSpamSection.php");
require_once("class.MagicSpamControl.php");

/**
 * This class controls the Spam Rules Section 
 */
class MagicSpamSpamRulesSection extends MagicSpamSection {

    /**
     * Flag to identify enabled rules
     * @var string $enControl
     */
    private static $enControl = "rules.enabled";

    /**
     * Flag to identify disabled rules
     * @var string $disControl
     */
    private static $disControl = "rules.disabled";

    /**
     * List containing the rule values
     * @var array $rules
     */
    private $rules;

    /**
     * Sets the template to spamrules, check enabled values, and initalizes arrays.
     */
    function __construct() {
        parent::__construct("spamrules.php");

        $defaultEnabled = array(
            "block_ip_in_addr", "check_dynamic_reverse_dns", "check_ip_reverse_dns", 
            "require_full_addr", "require_helo", "valid_helo_domain", "check_reverse_dns_list"
        );
       
        $enabled = MagicSpamControl::getMulti(self::$enControl, $defaultEnabled);

        $this->rules = array(
            "block_ip_in_addr" => array(
                "description" => "Block messages from IP (no domain)",
                "url" => "http://www.linuxmagic.com/best_practices/block_ip_in_addr.html",
            ), "check_dynamic_reverse_dns" => array(
                "description" => "Block Mail Servers on Dynamic/Dial-up Addresses",
                "url" => "http://www.linuxmagic.com/best_practices/check_dynamic_reverse_dns.html",
            ), "check_ip_reverse_dns" => array(
                "description" => "Perform reverse lookup check",
                "url" => "http://www.linuxmagic.com/best_practices/check_ip_reverse_dns.html",
            ), "check_reverse_dns_list" => array(
                "description" => "Block Mail Servers reported as Spam Source",
                "url" => "http://www.linuxmagic.com/best_practices/check_reverse_dns_list.html",
            ), "resolve_helo_domain" => array(
                "description" => "Confirm Server Identification Resolves (HELO)",
                "url" => "http://www.linuxmagic.com/best_practices/resolve_helo_domain.html",
            ), "require_full_addr" => array(
                "description" => "Strict address parsing",
                "url" => "http://www.linuxmagic.com/best_practices/require_full_address.html",
            ), "require_helo" => array(
                "description" => "Sending server must identify itself (HELO)",
                "url" => "http://www.linuxmagic.com/best_practices/require_helo.html",
            ), "valid_from_domain" => array(
                "description" => "Valid FROM domain",
                "url" => "http://www.linuxmagic.com/best_practices/valid_from_address.html",
            ), "valid_helo_domain" => array(
                "description" => "Server Identification must be valid (HELO)",
                "url" => "http://www.linuxmagic.com/best_practices/valid_helo_domain.html",
            ), "known_sender_forgery" => array(
                "description" => 'Matches if the sending IP is forging a known domain from a set of IPs valid for the domain.',
                'url' => "http://www.linuxmagic.com/best_practices/known_sender_forgery.html",
            ),
        );

        // set the default value for each rule
        foreach ($this->rules as $rule=>&$data) {
            $data["default"] = in_array($rule, $defaultEnabled);
            $data["enabled"] = in_array($rule, $enabled);
        }
    }

    /**
     * Prepares for the editing of rule state.  Check to ensure that an array of rules is
     * posted.  If so values are modified and set; otherwise, execeptions are thrown.
     */
    function prepare() {
        // Handle request to set a custom rejection notice
        if (array_key_exists("notice", $_POST)) {
            $cmd = REJECTIONS_MANAGER . " -a -n ";
            $cmd .= $_POST['rule'];
            $cmd .= " -m \"";
            $cmd .= $_POST['notice'];
            $cmd .= "\"";
            // TODO: add error handling for exit code of command - read from STDERR
            $pfh = popen($cmd, "r");
            pclose($pfh);
            $this->set("success", true);
        }
        // Handle request to reset to defaults
        if (array_key_exists("reset", $_POST)) {
            // remove the control files..
            if (file_exists(CONTROLS . self::$enControl)) {
                unlink(CONTROLS . self::$enControl);
            }
            if (file_exists(CONTROLS . self::$disControl)) {
                unlink(CONTROLS . self::$disControl);
            }
            // and set back to recommended.
            $defaultEnabled = array(
                "block_ip_in_addr", "check_dynamic_reverse_dns", "check_ip_reverse_dns",
                "require_full_addr", "require_helo", "valid_helo_domain"
            );

            // set the default value for each rule
            foreach ($this->rules as $rule=>&$data) {
                $data["default"] = in_array($rule, $defaultEnabled);
                $data["enabled"] = in_array($rule, $defaultEnabled);
            }
            // and ensure that we don't try to save over top.
            $_POST["rules"] = null;
            $this->set("success",true);
        }
        if (array_key_exists("rules", $_POST) && is_array($_POST["rules"])) {
            $enabled = array();
            $disabled = array();
            foreach ($_POST["rules"] as $rule=>$value) {
                if (array_key_exists($rule, $this->rules)) {
                    if ($value == "true") {
                        $enabled[] = $rule;
                    } else {
                        $disabled[] = $rule;
                    }
                } 
            }

            try {
                MagicSpamControl::setMulti(self::$enControl, $enabled);
                MagicSpamControl::setMulti(self::$disControl, $disabled);
                $this->set("success", true);

                // save changes to our local rules var
                foreach ($this->rules as $rule => &$data) {
                    $data["enabled"] = in_array($rule, $enabled);
                }
            } catch (Exception $e) {
                $this->set("error", true);
                $this->set("exception", $e);
            }
        }
    }

    /**
     * Sets rules to templateVar before rendering rules section output
     */
    function render() {
        $this->set("rules", $this->rules);

        parent::render();
    }
}

?>
